/**
 * C++ Implementation: textstyle
 *
 * Description:
 *
 * Author: Lorenzo Bettini <bettini@gnu.org>, (C) 2005
 * Copyright: See COPYING file that comes with this distribution
 */

#include "textstyle.h"
#include <boost/regex.hpp>

using namespace std;

TextStyle::TextStyle(const string &s) : repr((s.size() ? s : TEXT_VAR_TEXT)),
  invalid(true)
{
}

TextStyle::~TextStyle()
{
}

/**
 * The parts vector contains the string repr split in parts: those that
 * constant parts and those that represent $text and $style variables.
 * For instance, if repr == "<span class=\"$style\">$text</span>", then we
 * have:
 *   parts[0] == "<span class=\""
 *   parts[1] == ""
 *   parts[2] == "\">"
 *   parts[3] == ""
 *   parts[4] == "</span>"
 *
 * Then style_substitutions contains the index(es) of parts corresponding to
 * $style variable occurrences that will be substituted with values
 * and text_substitutions is the same but for $text occurrences.  Thus, in this
 * case:
 * style_substitutions[0] = 1
 * text_substitutions[0] = 3
 */
void
TextStyle::build_vectors()
{
  parts.clear();
  text_substitutions.clear();
  style_substitutions.clear();

  boost::regex e("(" STYLE_VAR ")|(" TEXT_VAR ")");
  boost::sregex_iterator i1(repr.begin(), repr.end(), e);
  boost::sregex_iterator i2;
  string suffix;

  if (i1 == i2)
    return ;

  int index = -1;
  for (boost::sregex_iterator it = i1; it != i2; ++it) {
    string prefix = it->prefix();
    if (prefix.size()) {
      parts.push_back(prefix);
      ++index;
    }

    ++index;
    if ((*it)[1].matched)
      style_substitutions.push_back(index);
    else
      text_substitutions.push_back(index);

    suffix = it->suffix();
    parts.push_back(""); // this will be set during "output"
    // with a value for $text or $style
  }

  if (suffix.size())
    parts.push_back(suffix);
}

/**
 * First substitute in the vector parts the values for $text and
 * $style and return a string with the substituted values.
 *
 * @param text
 * @param style
 * @return
 */
string
TextStyle::output(const string &text, const string &style)
{
  if (invalid) {
    build_vectors();
    invalid = false;
  }

  IndexVector::const_iterator indexIt;

  for (indexIt = text_substitutions.begin(); indexIt != text_substitutions.end(); ++indexIt)
    parts[*indexIt] = text;

  for (indexIt = style_substitutions.begin(); indexIt != style_substitutions.end(); ++indexIt)
    parts[*indexIt] = style;

  string ret;

  for (StringVector::const_iterator it = parts.begin(); it != parts.end(); ++it)
    ret += *it;

  return ret;
}

string
TextStyle::subst_style(const string &style)
{
  return output(TEXT_VAR_TEXT, style); // leave $text untouched
}

TextStyle
TextStyle::compose(const TextStyle &inner)
{
  return TextStyle(output(inner.toString()));
}

void
TextStyle::update(const string &inner)
{
  repr = output(inner);
  invalid = true;
}

void
TextStyle::update(const string &text, const string &style)
{
  repr = output(text, style);
  invalid = true;
}

void
TextStyle::update(const TextStyle &inner)
{
	update(inner.toString());
}

bool
TextStyle::containsStyleVar() const
{
    boost::regex style_exp(STYLE_VAR);
    return boost::regex_search(repr, style_exp);
}

bool
TextStyle::empty() const
{
  return repr == STYLE_VAR_TEXT || repr == TEXT_VAR_TEXT;
}
